import logging

LOGIN_URL = 'https://www.udemy.com/join/login-popup/?displayType=ajax&display_type=popup&showSkipButton=1&returnUrlAfterLogin=https%3A%2F%2Fwww.udemy.com%2F&next=https%3A%2F%2Fwww.udemy.com%2F&locale=en_US'
LOGIN_POPUP_URL = 'https://www.udemy.com/join/login-popup'
LOGOUT_URL = 'http://www.udemy.com/user/logout'
COURSE_TITLE_URL = 'https://www.udemy.com/api-2.0/courses/{course_id}?fields[course]=title'
COURSE_INFO_URL = 'https://www.udemy.com/api-2.0/courses/{course_id}/cached-subscriber-curriculum-items?fields[asset]=@min,title,filename,asset_type,external_url,length&fields[chapter]=@min,description,object_index,title,sort_order&fields[lecture]=@min,object_index,asset,supplementary_assets,sort_order,is_published,is_free&fields[quiz]=@min,object_index,title,sort_order,is_published&page_size=550'
GET_LECTURE_URL = 'https://www.udemy.com/api-2.0/users/me/subscribed-courses/{course_id}/lectures/{lecture_id}?fields[lecture]=asset,description,download_url&fields[asset]=asset_type,stream_urls,thumbnail_url,download_urls'
ATTACHMENT_URL = 'https://www.udemy.com/api-2.0/users/me/subscribed-courses/{course_id}/lectures/{lecture_id}/supplementary-assets/{attachment_id}?fields[asset]=download_urls'

__title__ = 'udemydl'
__author__ = 'Andrey Larin'
__email__ = 'lestex@gmail.com'
__version__ = '0.5.0'

logger = logging.getLogger(__title__)



from . import __title__
from .argparser import parser
from .utils import sys_info
from .udemy_download import UdemyDownload
from .exceptions import UdemyException
import logging
import getpass
import os
import sys

logger = logging.getLogger(__title__)

class Cli:

    LOG_FORMAT_CONSOLE = '%(levelname)-8s %(message)s'

    def __init__(self):
        self.argparser = parser

    def __get_log_handler(self, error_level, formatter,
                    handler, handler_args={}):
        log_handler = handler(**handler_args)
        log_handler.setLevel(error_level)
        log_handler.setFormatter(logging.Formatter(formatter))
        return log_handler

    def __get_console_log_handler(self, error_level):
        return self.__get_log_handler(
            error_level, self.LOG_FORMAT_CONSOLE,
            logging.StreamHandler
        )

    def __init_logger(self, error_level=logging.INFO):
        logger.setLevel(error_level)
        logger.addHandler(self.__get_console_log_handler(error_level))
    
    def __generate_sys_info_log(self, sys_info):
        logger.debug('Running on:')
        logger.debug('Python: {}'.format(sys_info['python']))
        logger.debug('Platform: {}'.format(sys_info['platform']))
        logger.debug('OS: {}'.format(sys_info['os']))

    def run(self):
        args = vars(self.argparser.parse_args())

        link = args['link']
        username = args['username']
        password = args['password']
        debug = args['debug']
        quality = args['video_quality']
        
        if not username:
            username = input("Username / Email : ")

        if not password:
            password = getpass.getpass()            

        sys_information = sys_info()

        if debug:
            self.__init_logger(logging.DEBUG)
            self.__generate_sys_info_log(sys_information)
        else:
            self.__init_logger()

        if args['output']:
            output_dest = os.path.normpath(args['output'])            
        else:
            course_slug = link.rsplit('/', 1)[1]
            output_dest = os.path.normpath(course_slug)           

        output_dir = os.path.abspath(output_dest)        
        logger.debug('Downloading course to: %s', output_dir)
        logger.debug('Downloading with quality: %s', quality)

        try:
            with UdemyDownload(link, username, password, output_dir, quality) as ud:                
                ud.analyze()
                ud.download()
        except UdemyException as ue:
            logger.error(ue.args[0])
        except KeyboardInterrupt:
            logger.error('User interrupted the process, exiting...')
        except Exception as e:
            logger.error('Unknown Exception')
            logger.exception(e)
        finally:
            sys.exit(1)



from . import __title__
from .session import session
from .exceptions import UdemyException
from .course import Course
from .downloader import Downloader
from . import LOGIN_URL, LOGIN_POPUP_URL, LOGOUT_URL
from . import logger
import re

class UdemyDownload(object):
    def __init__(self, course_url, username, password, output_dir, quality):
        self.username = username
        self.password = password
        self.course_url = course_url
        self.output_dir = output_dir
        self.quality = quality
        self.session = session
        self.data_links = None
    
    def __enter__(self):
        self.login()
        return self

    def __exit__(self, *args):
        self.logout()
    
    def __get_csrf_token(self):
        response = self.session.get(LOGIN_POPUP_URL)
        match = re.search("name=\'csrfmiddlewaretoken\' value=\'(.*)\'", response.text)
        return match.group(1)
    
    def login(self):
        logger.info('Trying to log in ...')        
        csrf_token = self.__get_csrf_token()
        payload = {'isSubmitted': 1, 'email': self.username, 'password': self.password,
                   'displayType': 'ajax', 'csrfmiddlewaretoken': csrf_token}
        response = self.session.post(LOGIN_URL, payload)

        access_token = response.cookies.get('access_token')
        client_id = response.cookies.get('client_id')

        response_text = response.text        
        
        if 'check your email and password' in response_text:            
            raise UdemyException('Wrong Username or Password!')

        elif access_token is None:            
            raise UdemyException('Couldn\'t fetch token!')            

        elif 'error' in response_text:            
            raise UdemyException('Found error in login page')

        self.session.set_auth_headers(access_token, client_id)
        logger.info("Login success.")


    def logout(self):
        self.session.get(LOGOUT_URL)
        logger.info('Logged Out!')        

    def analyze(self):
        logger.info('Analyzing the course ...')
        course = Course(self.course_url, self.session)
        self.data_links = course.get_course_data(self.quality)
        #logger.info('data links: %s', self.data_links)

    def download(self):
        logger.info('Downloading files ...')
        Downloader(self.data_links, self.output_dir).download()        
        logger.info('Downloaded all files.')
        


import argparse
from udemy_dl import __title__

parser = argparse.ArgumentParser(description='Download courses from Udemy.com', prog=__title__)
parser.add_argument('--debug', help='Enable debug mode', action='store_const', const=True, default=False)
parser.add_argument('link', help='Link for udemy course', action='store')
parser.add_argument('-u', '--username', help='Username / Email', default=None, action='store')
parser.add_argument('-p', '--password', help='Password', default=None, action='store')
parser.add_argument('-o', '--output', help='Output directory', default=None, action='store')
parser.add_argument('-q', '--video-quality', help='Select video quality 720, 480 (default is 720)', default='720', action='store')



#!/usr/bin/env python
from setuptools import setup
from udemy_dl import __version__, __author__, __email__
import os

requirements = [pkg.split('=')[0] for pkg in open('requirements.txt').readlines()]

description = 'Download videos from Udemy for personal offline use'

this_directory = os.path.abspath(os.path.dirname(__file__))

try:
    with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
except OSError:
    print("Couldn't find README.md")

classifiers = ['Environment :: Console',
               'Programming Language :: Python :: 2',
               'Programming Language :: Python :: 3',
               'Topic :: Multimedia :: Video',]

setup(name='udemydl',
    version=__version__,
    description=description,
    author=__author__,
    author_email=__email__,
    url='https://github.com/lestex/udemydl',      
    install_requires=requirements,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=['udemy_dl'],
    classifiers=classifiers,
    entry_points={
        'console_scripts': [
            '{}=udemy_dl.__main__:main'.format('udemydl')
        ]
    }
)



import requests

class Session:
    headers = {
        'Host': 'www.udemy.com',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:39.0) Gecko/20100101 Firefox/66.0',
        'X-Requested-With': 'XMLHttpRequest',
        'Referer': 'https://www.udemy.com/join/login-popup',
        'Accept': '*/*',
        'Accept-Language': 'en-US,en;q=0.5',
        'Accept-Encoding': 'gzip, deflate, br',
        'Connection': 'keep-alive'
    }

    def __init__(self):
        self.session = requests.sessions.Session()
        
    def set_auth_headers(self, access_token, client_id):
        self.headers['Authorization'] = 'Bearer {}'.format(access_token)

    def get(self, url):
        return self.session.get(url, headers=self.headers)

    def post(self, url, data):
        return self.session.post(url, data, headers=self.headers)

session = Session()
